FFmpeg
Loading...
Searching...
No Matches
lls.c
Go to the documentation of this file.
1/*
2 * This file is part of FFmpeg.
3 *
4 * FFmpeg is free software; you can redistribute it and/or
5 * modify it under the terms of the GNU Lesser General Public
6 * License as published by the Free Software Foundation; either
7 * version 2.1 of the License, or (at your option) any later version.
8 *
9 * FFmpeg is distributed in the hope that it will be useful,
10 * but WITHOUT ANY WARRANTY; without even the implied warranty of
11 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
12 * Lesser General Public License for more details.
13 *
14 * You should have received a copy of the GNU Lesser General Public
15 * License along with FFmpeg; if not, write to the Free Software
16 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
17 */
18
19#include <limits.h>
20#include <stdio.h>
21
22#include "libavutil/internal.h"
23#include "libavutil/lfg.h"
24#include "libavutil/lls.h"
26
27/* Values below the printed precision are rounding noise whose sign varies
28 * between platforms, so print them as zero. */
29static double clean(double v)
30{
31 return fabs(v) < 1e-9 ? 0.0 : v;
32}
33
34int main(void)
35{
36 LLSModel m;
37 int i, order;
38 AVLFG lfg;
39
40 av_lfg_init(&lfg, 1);
41 avpriv_init_lls(&m, 3);
42
43 for (i = 0; i < 100; i++) {
44 LOCAL_ALIGNED(32, double, var, [4]);
45 double eval;
46
47 var[0] = (av_lfg_get(&lfg) / (double) UINT_MAX - 0.5) * 2;
48 var[1] = var[0] + av_lfg_get(&lfg) / (double) UINT_MAX - 0.5;
49 var[2] = var[1] + av_lfg_get(&lfg) / (double) UINT_MAX - 0.5;
50 var[3] = var[2] + av_lfg_get(&lfg) / (double) UINT_MAX - 0.5;
51 m.update_lls(&m, var);
52 avpriv_solve_lls(&m, 0.001, 0);
53 for (order = 0; order < 3; order++) {
54 eval = m.evaluate_lls(&m, var + 1, order);
55 printf("real:%9f order:%d pred:%9f var:%f coeffs:%f %9f %9f\n",
56 var[0], order, eval,
57 sqrt(FFMAX(m.variance[order], 0) / (i + 1)),
58 clean(m.coeff[order][0]), clean(m.coeff[order][1]),
59 clean(m.coeff[order][2]));
60 }
61 }
62 return 0;
63}
#define i(width, name, range_min, range_max)
Definition cbs_h264.c:63
__device__ int printf(const char *,...)
static __device__ float fabs(float a)
av_cold void av_lfg_init(AVLFG *c, unsigned int seed)
Definition lfg.c:32
static unsigned int av_lfg_get(AVLFG *c)
Get the next random unsigned 32-bit number using an ALFG.
Definition lfg.h:53
common internal API header
av_cold void avpriv_init_lls(LLSModel *m, int indep_count)
Definition lls.c:109
void avpriv_solve_lls(LLSModel *m, double threshold, unsigned short min_order)
Definition lls.c:47
static double clean(double v)
Definition lls.c:29
int main(void)
Definition lls.c:34
#define FFMAX(a, b)
Definition macros.h:47
#define LOCAL_ALIGNED(a, t, v,...)
Context structure for the Lagged Fibonacci PRNG.
Definition lfg.h:33
Linear least squares model.
Definition lls.h:37
void(* update_lls)(struct LLSModel *m, const double *var)
Take the outer-product of var[] with itself, and add to the covariance matrix.
Definition lls.h:49
double variance[MAX_VARS]
Definition lls.h:40
double(* evaluate_lls)(struct LLSModel *m, const double *var, int order)
Inner product of var[] and the LPC coefs.
Definition lls.h:56
double coeff[MAX_VARS][MAX_VARS]
Definition lls.h:39